package com.supermap.dynamicchart;

/**
 * <p>
 * Title: Dynamic chart
 * </p>
 *
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------Statement----------------------------
 * This is for SuperMap iMobile for Android Sample Code
 * Copyright: SuperMap Software Co., Ltd.
 * ----------------------------------------------------------------
 * ----------------------------SuperMap iMobile
 * Demo---------------------------
 *
 * 1. Introduction:
 *   	Display dynamic chart.
 *
 * 2. Demo data:
 * 		Data: "../SampleData/DynamicChartData/"
 *      Map: "China400.smwu", "China.udb", "China.udd"
 *      License: "../SuperMap/License/"
 *
 * 3. Members:
 *    recordset.seekID();				Method
 *    recordset.getFieldValue();		Method
 *    data.setColor();					Method
 *    data.setGeoID();					Method
 *    data.setLabel();					Method
 *    data.setValue();					Method
 *    m_pieChart.addData();				Method
 *    m_pieChart.setChartTitle();		Method
 *    m_pieChart.getLegendView();		Method
 *    m_pieChart.reLayout();			Method
 *    m_lineChart.addXLabel();			Method
 *
 * 4. Function Display
 *   (1)Create and display the line chart
 *   (2)Create and display pie chart
 *   (3)Create and display the bar graph
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p>
 *
 * <p>
 * Company: SuperMap Software Co., Ltd.
 * </p>
 *
 */
import android.app.Activity;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.widget.RelativeLayout;
import com.supermap.data.CursorType;
import com.supermap.data.DatasetVector;
import com.supermap.data.Environment;
import com.supermap.data.Recordset;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.data.WorkspaceType;
import com.supermap.mapping.Action;
import com.supermap.mapping.GeometrySelectedEvent;
import com.supermap.mapping.GeometrySelectedListener;
import com.supermap.mapping.Layer;
import com.supermap.mapping.Layers;
import com.supermap.mapping.MapControl;
import com.supermap.mapping.MapView;
import com.supermap.mapping.imChart.BarChartData;
import com.supermap.mapping.imChart.BarChart;
import com.supermap.mapping.imChart.ChartsView;
import com.supermap.mapping.imChart.InstrumentChart;
import com.supermap.mapping.imChart.LegendView;
import com.supermap.mapping.imChart.LineChartData;
import com.supermap.mapping.imChart.LineChart;
import com.supermap.mapping.imChart.PieChartData;
import com.supermap.mapping.imChart.PieChart;

import java.util.ArrayList;

/**
 * Created by Administrator on 2017/9/12.
 */

public class MainActivity extends Activity {
    final String sdcard = android.os.Environment.getExternalStorageDirectory().getAbsolutePath().toString();
    private MapControl m_mapControl = null;
    private Workspace m_workSpace = null;
    private MapView m_mapView = null;
    private ChartsView m_pieChart = null;
    private ChartsView m_lineChart = null;
    private ChartsView m_barChart = null;
    private InstrumentChart m_instrumentChart = null;
    private RelativeLayout m_layout = null;
    private Layers m_layers = null;
    private Layer m_layer = null;
    boolean m_bShowPieChart = false;
    boolean m_bShowLineChart = false;
    boolean m_bShowBarChart = false;
    String queryName;
    int houseValue;
    ArrayList<String> strName = new ArrayList<String>();
    ArrayList<Integer> strValue = new ArrayList<Integer>();
    int id;

    private enum CHART_STATE {STATE_LINECHART, STATE_BARCHART,
        STATE_PIECHART, STATE_NULL,STATE_DIALCHART};
    private CHART_STATE m_lastChart = CHART_STATE.STATE_NULL;
    private CHART_STATE m_curChart = CHART_STATE.STATE_NULL;
    private CHART_STATE m_dChart = CHART_STATE.STATE_NULL;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        //Use the OpenGL mode to display
        Environment.setOpenGLMode(true);
        //Set the license path
        Environment.setLicensePath(sdcard + "/SuperMap/license/");
        //Component functions only be used after Environment initialization
        Environment.initialization(this);
        setContentView(R.layout.activity_main);

        //Open the workspace
        m_workSpace = new Workspace();
        WorkspaceConnectionInfo info = new WorkspaceConnectionInfo();
        info.setServer(sdcard+"/SampleData/DynamicChartData/China400.smwu");
        info.setType(WorkspaceType.SMWU);
        boolean result = m_workSpace.open(info);

        if (!result) {
            System.out.println("Failed to open the workspace");
            return;
        }

        //The map display control is associated to the workspace
        m_mapView = (MapView)findViewById(R.id.Map_view);
        m_mapControl =  m_mapView.getMapControl();
        m_mapControl.getMap().setWorkspace(m_workSpace);
        //Open the map in the workspace
        String mapName = m_workSpace.getMaps().get(0);
        result = m_mapControl.getMap().open(mapName);
        if (!result) {
            System.out.println("Failed to open the map");
            return;
        }

        //Initialize the chart
        m_pieChart = (ChartsView) findViewById(R.id.pieChart);
        m_lineChart = (ChartsView) findViewById(R.id.lineChart);
        m_barChart = ((ChartsView) findViewById(R.id.barChart));
        m_instrumentChart = (InstrumentChart) findViewById(R.id.diaChart);

        m_layers = m_mapControl.getMap().getLayers();
        m_mapControl.setAction(Action.SELECT);
        m_layer = m_layers.get("China_Provinces@China");	//Get the layer to create the dynamic chart

        m_pieChart.setLayer(m_layer);
        m_lineChart.setLayer(m_layer);
        m_barChart.setLayer(m_layer);
        m_instrumentChart.setLayer(m_layer);
		m_barChart.setControl(m_mapView.getMapControl());
        m_pieChart.setControl(m_mapView.getMapControl());
        m_lineChart.setControl(m_mapView.getMapControl());
        m_layer.setEditable(false);
        m_layer.setSelectable(true);

        //Refresh the map
        m_mapControl.getMap().refresh();

        //The chart window is invisible
        m_layout = (RelativeLayout) findViewById(R.id.ChartView);
        m_layout.setVisibility(View.INVISIBLE);

        //Create the dynamic chart
        CreatePieChart();		//Pie
        CreateLineChart();		//Line
        CreateBarChart();		//Bar
        CreateDiaChart();       //Dashboard

        new Thread(new Runnable() {
            @Override
            public void run() {
                // TODO Auto-generated method stub
                DatasetVector data = (DatasetVector) m_layer.getDataset();
                Recordset recordset = data.getRecordset(false, CursorType.STATIC);
                recordset.moveFirst();
                while (!recordset.isEOF()) {
                    strName.add((String) recordset.getFieldValue("省级行政单位"));
                    strValue.add((Integer) recordset.getFieldValue("House_Ave_Price_2012"));
                    recordset.moveNext();
                }
            }
        }).start();
    }
    private void CreateDiaChart() {
        // TODO Auto-generated method stub
        ((InstrumentChart) m_instrumentChart).setMinValue(0);
        ((InstrumentChart) m_instrumentChart).setMaxValue(200);
        ((InstrumentChart) m_instrumentChart).setPointColor(Color.BLACK);
        ((InstrumentChart) m_instrumentChart).setPoint("", 0);

        m_mapControl.addGeometrySelectedListener(new GeometrySelectedListener() {
            @Override
            public void geometrySelected(GeometrySelectedEvent event) {
                id = event.getGeometryID();
                queryName = strName.get(id - 1);
                if(strValue.get(id - 1)!=null){
                    houseValue = strValue.get(id - 1);
                }
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        ((InstrumentChart) m_instrumentChart).setPoint("", houseValue/100);
                        ((InstrumentChart) m_instrumentChart).setChartTitle(queryName+"房价");
                    }
                });
            }
            @Override
            public void geometryMultiSelected(ArrayList<GeometrySelectedEvent> events) {
            }
        });

    }

    //"Dashboard"按钮
    public void btnDiaChart_Click(View view){

        showDiaChart();
    }


    //"Bar"按钮
    public void btnBarChart_Click(View view){

        showBarChart();
    }

    //"Line"按钮
    public void btnLineChart_Click(View view){

        showLineChart();
    }

    //"Pie"按钮
    public void btnPieChart_Click(View view){

        showPieChart();
    }

    //Create the pie chart
    public void CreatePieChart(){

        //Get the dataset corresponding to the chart layer
        DatasetVector datasetVector = (DatasetVector)m_layer.getDataset();
        //Get the recordset
        Recordset recordset = datasetVector.getRecordset(false, CursorType.STATIC);
        //Beijing, Tianjin, Shanghai and Chongqing ID
        int [] IDs = {30, 22, 29, 27};
        //Set the display color
        int [] colors =  {
                Color.rgb(192, 255, 140), Color.rgb(255, 247, 140), Color.rgb(255, 208, 140),
                Color.rgb(140, 234, 255), Color.rgb(255, 140, 157)
        };
        //The chart label value
        String [] labels = {"Beijing", "Tianjin", "Shanghai", "Chongqing"};

        int i = 0;
        String field_HousePrice = "House_Ave_Price_2012";	//The dataset attribute name
        boolean isTrue = false;
        for(i=0; i<IDs.length; i++){
            isTrue = recordset.seekID(IDs[i]);	//Get the corresponding record through the ID
            if(isTrue){
                Object value = recordset.getFieldValue(field_HousePrice);	//Get the corresponding attribute

                PieChartData data = new PieChartData();
                data.setColor(colors[i]);	//Set the icon item color
                data.setGeoID(IDs[i]);		//Set the chart item SmID of the relationship geometry object
                data.setLabel(labels[i]);	//Set chart item label
                data.setValue((Integer)value);	//Set the chart item value
                ((PieChart)m_pieChart).addData(data);	//Add chart data
            }
        }
        m_pieChart.setChartTitle("Comparison of average house prices in municipalities in 2012");		//Set the chart title

        LegendView legendView = m_pieChart.getLegendView();//Get the legend LegendView
        legendView.setColumnWidth(legendView.getColumnWidth() + 150);	//Set the legend item width
        legendView.setNumColumns(5);
        m_pieChart.reLayout();	//Need to call this method to layout after the modification of the layout

        recordset.close();	//Close the recordset
        recordset.dispose();//Release the resources
    }

    public void CreateLineChart(){

        //Get the corresponding dataset used to create the chart layer
        DatasetVector datasetVector = (DatasetVector)m_layer.getDataset();
        //Get the recordset
        Recordset recordset = datasetVector.getRecordset(false, CursorType.STATIC);
        //Beijing, Tianjin, Shanghai and Chongqing ID
        int [] IDs = {30, 22, 29, 27};
        //Set the display color
        int [] colors =  {
                Color.rgb(192, 255, 140), Color.rgb(255, 247, 140), Color.rgb(255, 208, 140),
                Color.rgb(140, 234, 255), Color.rgb(255, 140, 157)
        };
        //The chart label value
        String [] labels = {"Beijing", "Tianjing", "Shanghai", "Chongqing"};

        String field_PoP = "Popu_";
        int [] years = {2002, 2010, 2011, 2012};
        boolean isTrue = false;
        for(int i=0; i<IDs.length; i++){
            isTrue = recordset.seekID(IDs[i]);	//Get the corresponding records through the ID
            if(isTrue){
                LineChartData lineChartData = new LineChartData();
                lineChartData.setColor(colors[i]);	//Set the chart item color
                lineChartData.setGeoID(IDs[i]);		//Set the chart item geometry object SmID
                lineChartData.setLabel(labels[i]);	//Set the chart item label

                for(int j=0; j<years.length; j++){
                    Object value = recordset.getFieldValue(field_PoP + years[j]); //Get the corresponding attributes
                    lineChartData.addValue((Integer) value);		//Add the value
                }
                ((LineChart)m_lineChart).addData(lineChartData);	//Add the chart data
            }
        }

        for(int i=0; i<years.length; i++){
            ((LineChart)m_lineChart).addXLabel(years[i]+"");	//Add the X axis labels
        }

        m_lineChart.setChartTitle("Comparison of population trends in municipality");	//Set the chart title

        LegendView legendView = m_lineChart.getLegendView();	//Get the legend LegendView
        legendView.setColumnWidth(legendView.getColumnWidth() + 150);//Set the legend item width
        legendView.setNumColumns(5);
        m_lineChart.reLayout();

        recordset.close();	//Close the recordset
        recordset.dispose();//Release the resources
    }

    public void CreateBarChart(){

        //Get the corresponding dataset used to create the chart layer
        DatasetVector datasetVector = (DatasetVector)m_layer.getDataset();
        //Get the recordset
        Recordset recordset = datasetVector.getRecordset(false, CursorType.STATIC);
        //Beijing, Tianjin, Shanghai and Chongqing ID
        int [] IDs = {30, 22, 29, 27};
        //The color of the bar chart
        int [] colors =  {
                Color.rgb(192, 255, 140), Color.rgb(255, 247, 140)};
        //The chart label value
        String [] labels = {"Beijing", "Tianjing", "Shanghai", "Chongqing"};

        String field_University = "university_num_2012";
        String field_Museum = "museum_num_2012";
        String field_ProvinceName = "NAME";
        boolean isTrue = false;
        for(int i=0; i<IDs.length; i++){

            isTrue = recordset.seekID(IDs[i]);	//Get the records through the ID
            if(isTrue){

                BarChartData barChartData = new BarChartData();
                Object name = recordset.getFieldValue(field_ProvinceName);
                if(name == null){
                    recordset.moveNext();
                    continue;
                }

                Object value = recordset.getFieldValue(field_University);
                if(value == null){
                    recordset.moveNext();
                    continue;
                }

                barChartData.addData("University", colors[0], (Short) value);
                value = recordset.getFieldValue(field_Museum);
                if(value == null){
                    recordset.moveNext();
                    continue;
                }

                barChartData.setGeoID(IDs[i]);
                barChartData.addData("Library", colors[1], (Short) value);
                ((BarChart) m_barChart).addData(barChartData);
                ((BarChart)m_barChart).addXLabel((String)name);
            }
        }
        m_barChart.setChartTitle("Contrast of number of university and libraries in municipalities in 2012");			//Set the chart title

        LegendView legendView = m_barChart.getLegendView();					//Get the legend LegendView
        legendView.setColumnWidth(legendView.getColumnWidth() + 180);		////Set the legend item width
        legendView.setNumColumns(5);//Set the legend item width and height
        m_barChart.reLayout();
        recordset.close();	//Close the recordset
        recordset.dispose();//Release the resources
    }

    public void showBarChart(){

        if(m_curChart == CHART_STATE.STATE_BARCHART){	//Close bar chart.

            m_layout.setVisibility(View.INVISIBLE);
            m_barChart.setVisibility(View.INVISIBLE);

            m_lastChart = m_curChart;
            m_curChart = CHART_STATE.STATE_NULL;
        }
        else{ //Open bar chart.
            m_layout.setVisibility(View.VISIBLE);
            m_pieChart.setVisibility(View.INVISIBLE);
            m_barChart.setVisibility(View.VISIBLE);
            m_lineChart.setVisibility(View.INVISIBLE);
            m_instrumentChart.setVisibility(View.INVISIBLE);
            m_lastChart = m_curChart;
            m_curChart = CHART_STATE.STATE_BARCHART;
        }
    }

    public void showLineChart(){

        if( m_curChart == CHART_STATE.STATE_LINECHART){	//Close line chart

            m_layout.setVisibility(View.INVISIBLE);
            m_lineChart.setVisibility(View.INVISIBLE);

            m_lastChart = m_curChart;
            m_curChart = CHART_STATE.STATE_NULL;
        }
        else{//Open line chart
            m_layout.setVisibility(View.VISIBLE);
            m_pieChart.setVisibility(View.INVISIBLE);
            m_barChart.setVisibility(View.INVISIBLE);
            m_lineChart.setVisibility(View.VISIBLE);
            m_instrumentChart.setVisibility(View.INVISIBLE);

            m_lastChart = m_curChart;
            m_curChart = CHART_STATE.STATE_LINECHART;
        }
    }

    public void showPieChart(){

        if(m_curChart == CHART_STATE.STATE_PIECHART){//Close pie chart

            m_layout.setVisibility(View.INVISIBLE);
            m_pieChart.setVisibility(View.INVISIBLE);

            m_lastChart = m_curChart;
            m_curChart = CHART_STATE.STATE_NULL;
        }
        else{
            m_layout.setVisibility(View.VISIBLE);
            m_pieChart.setVisibility(View.VISIBLE);
            m_barChart.setVisibility(View.INVISIBLE);
            m_lineChart.setVisibility(View.INVISIBLE);
            m_instrumentChart.setVisibility(View.INVISIBLE);

            m_lastChart = m_curChart;
            m_curChart = CHART_STATE.STATE_PIECHART;
        }
    }

    private void showDiaChart() {
        // TODO Auto-generated method stub
        if(m_curChart == CHART_STATE.STATE_DIALCHART){

            m_layout.setVisibility(View.INVISIBLE);
            m_instrumentChart.setVisibility(View.INVISIBLE);
            m_lastChart = m_curChart;
            m_curChart = CHART_STATE.STATE_NULL;
        }
        else{
            m_layout.setVisibility(View.VISIBLE);
            m_pieChart.setVisibility(View.INVISIBLE);
            m_barChart.setVisibility(View.INVISIBLE);
            m_lineChart.setVisibility(View.INVISIBLE);
            m_instrumentChart.setVisibility(View.VISIBLE);
            m_lastChart = m_curChart;
            m_curChart = CHART_STATE.STATE_DIALCHART;
        }
    }


}